@@ -1,5 +1,5 @@
 package Number::MuPhone::Config;
-use 5.012;
+use strictures 2;
 use Exporter;
 use Carp;
 
@@ -8,26 +8,28 @@ use Carp;
 # - ENV{MUPHONE_CONF_FILEPATH}
 # - $ENV{HOME}/.muphone_conf.yaml
 
-# your config MUST contain the elements 'default_country' and 'dialer'
+# your config MUST contain the elements 'default_country', 'countries' and 'dialer'
 
-# default basic config 
+# default empty config 
 $Number::MuPhone::Config::config = {
   # two letter country code
-  # if a number is supplied without a country attr and doesn't begin with +CODE
+  # if a number is supplied withoput a country attr and doesn't begin with +CODE
   # try to parse the number as if it's in this country
-  default_country => 'US',
-  
+  default_country => '',
+
+  countries => {
+    US => {
+
+    },
+  },
   dialer => {
     # what is the pause / wait for extension character for your dialer
     pause => ','
   },
+  is_default => 1,
 };
 
-my $home_conf = $ENV{HOME}
-                ? "$ENV{HOME}/.muphone_conf.yaml"
-                : '';
-
-my $file_path = $ENV{MUPHONE_CONF_FILEPATH} || $home_conf;
+my $file_path = $ENV{MUPHONE_CONF_FILEPATH} || $ENV{HOME}.'/.muphone_conf.yaml';
 
 if ( -f $file_path ) {
   # only try and load YMAL::XS if config exists
